/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
import { Component, OnInit, AfterViewInit, OnChanges, Input, Output, HostListener, EventEmitter, ViewChild } from "@angular/core";
import { FormGroup } from "@angular/forms";
import { EmojiService } from "../emoji.service";
export class EmojiInputComponent {
    /**
     * @param {?} emojiService
     */
    constructor(emojiService) {
        this.emojiService = emojiService;
        this.popupAnchor = 'top';
        this.inputClass = '';
        this.searchClass = '';
        this.onEnter = () => {
        };
        this.autofocus = false;
        this.closeAfterSelection = true;
        this.placeHolder = '';
        this.modelChange = new EventEmitter();
        this.setPopupAction = new EventEmitter();
        this.blur = new EventEmitter();
        this.focus = new EventEmitter();
        this.keyup = new EventEmitter();
        this.emojiClick = new EventEmitter();
        this.input = '';
        this.filterEmojis = '';
        this.popupOpen = false;
        this.lastCursorPosition = 0;
    }
    /**
     * @param {?} $event
     * @return {?}
     */
    clickedOutside($event) {
        this.popupOpen = false;
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        if (this.setPopupAction) {
            this.setPopupAction.emit((status) => {
                this.openPopup(status);
            });
        }
        this.allEmojis = this.emojiService.getAll();
        this.clean();
    }
    /**
     * @return {?}
     */
    ngAfterViewInit() {
        if (this.autofocus) {
            if (this.textArea) {
                this.textareaEl.nativeElement.focus();
            }
            else {
                this.inputEl.nativeElement.focus();
            }
        }
    }
    /**
     * @return {?}
     */
    ngOnChanges() {
        if (this.model !== this.input) {
            this.input = this.model;
        }
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onKeyup(event) {
        this.updateCursor();
        if (this.keyup) {
            this.keyup.emit(event);
        }
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onBlur(event) {
        this.updateCursor();
        if (this.blur) {
            this.blur.emit(event);
        }
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onFocus(event) {
        this.updateCursor();
        if (this.focus) {
            this.focus.emit(event);
        }
    }
    /**
     * @return {?}
     */
    clean() {
        this.filterEmojis = '';
        this.filteredEmojis = this.getFilteredEmojis();
    }
    /**
     * @param {?} $event
     * @param {?=} status
     * @return {?}
     */
    openPopup($event, status = null) {
        if (status === null) {
            this.popupOpen = !this.popupOpen;
        }
        else {
            this.popupOpen = status;
        }
        $event.preventDefault();
        $event.stopPropagation(); // <- that will stop propagation on lower layers
    }
    /**
     * @return {?}
     */
    updateFilteredEmojis() {
        this.filteredEmojis = this.getFilteredEmojis();
    }
    /**
     * @return {?}
     */
    getFilteredEmojis() {
        return this.allEmojis.filter((e) => {
            if (this.filterEmojis === '') {
                return true;
            }
            else {
                for (let /** @type {?} */ alias of e.aliases) {
                    if (alias.includes(this.filterEmojis)) {
                        return true;
                    }
                }
            }
            return false;
        });
    }
    /**
     * @param {?} e
     * @return {?}
     */
    onEmojiClick(e) {
        this.input = this.input.substr(0, this.lastCursorPosition) + e + this.input.substr(this.lastCursorPosition);
        this.modelChange.emit(this.input);
        this.emojiClick.emit(e);
        if (this.closeAfterSelection) {
            this.popupOpen = false;
            this.clean();
        }
        else {
            this.lastCursorPosition += 2;
        }
    }
    /**
     * @param {?} newValue
     * @return {?}
     */
    onChange(newValue) {
        if (newValue) {
            this.input = this.emojiService.emojify(newValue);
            this.model = this.input;
            this.modelChange.emit(this.input);
        }
    }
    /**
     * @return {?}
     */
    updateCursor() {
        if (this.textArea) {
            this.lastCursorPosition = this.textareaEl.nativeElement.selectionStart;
        }
        else {
            this.lastCursorPosition = this.inputEl.nativeElement.selectionStart;
        }
    }
}
EmojiInputComponent.decorators = [
    { type: Component, args: [{
                selector: 'emoji-input',
                template: `
    <ng-template [ngIf]="textArea">
      <div [formGroup]="group">
        <mat-form-field>
          <textarea #textareaEl matInput matTextareaAutosize
                name="text"
                formControlName="title"
                placeholder="{{placeHolder}}"
                minRows = "1"
                [ngClass]="[inputClass]"
                (keyup)="onKeyup($event)"
                (keyup.enter)="onEnter()"
                (blur)="onBlur($event)"
                (focus)="onFocus($event)"
                (ngModelChange)="onChange($event)"
                [(ngModel)]="input">
          </textarea>
        </mat-form-field>
        <button type="button" name="button" class="add-emoji" (click)="openPopup($event)">☺</button>
      </div>
    </ng-template>
    <ng-template [ngIf]="!textArea">
      <div [formGroup]="group">
        <mat-form-field>
          <input #inputEl matInput
                 type="text"
                 autocomplete="on"
                 formControlName="title"
                 placeholder="{{placeHolder}}"
                 [ngClass]="[inputClass]"
                 (keyup)="onKeyup($event)"
                 (keyup.enter)="onEnter()"
                 (blur)="onBlur($event)"
                 (focus)="onFocus($event)"
                 (ngModelChange)="onChange($event)"
                 [(ngModel)]="input"
                 required/>
        </mat-form-field>
        <button type="button" name="button" class="add-emoji" (click)="openPopup($event)">☺</button>
      </div>
    </ng-template>
    <div class="emoji-search"
         [ngClass]="[popupAnchor, searchClass]"
         [hidden]="!popupOpen"
         (click)="$event.stopPropagation()">
      <div class="search-header">
        <input type="search" placeholder="Search..."
               [(ngModel)]="filterEmojis"
               (ngModelChange)="updateFilteredEmojis()"/>
      </div>
      <div class="emojis-container">
        <span *ngFor="let emoji of filteredEmojis"
              (click)="onEmojiClick(emoji.emoji)"
              title="{{emoji.aliases[0]}}">
          {{emoji.emoji}}
        </span>
      </div>
    </div>
  `,
                styles: [`
    :host {
      display: block;
      position: relative;
    }

    :host .emoji-search {
      background-color: #fff;
      border: 1px solid #ccc;
      border-radius: 4px;
      box-shadow: 0 1px 5px rgba(0, 0, 0, 0.15);
      height: auto;
      line-height: 1.5;
      position: absolute;
      right: 0;
      width: 100%;
      max-width: 285px;
      display: flex;
      flex-direction: column;
      z-index: 100;
    }

    :host .emoji-search[hidden] {
      display: none;
    }

    :host .emoji-search.bottom {
      top: -202px;
    }

    :host .mat-form-field {
      width: 100%;
    }
    
    :host .add-emoji {
      font-size: 25px;
      vertical-align: middle;
      position: absolute;
      top: 16px;
      right: -20px;
      width: 20px;
    }

    :host .emoji-search input {
      border-radius: 4px;
      font-size: 10px;
      padding: 4px 8px;
      margin: 0;
      height: 30px;
    }

    :host .emoji-search .search-header {
      background-color: #eee;
      border-bottom: 1px solid #ccc;
      border-radius: 4px 4px 0 0;
      padding: 4px 8px;
      width: 100%;
    }

    :host .emoji-search .emojis-container {
      border-radius: 0 0 4px 4px;
      max-height: 160px;
      padding: 5px 12px;
      overflow: auto;
      overflow-x: hidden;
      flex: 1;
      display: flex;
      flex-wrap: wrap;
      justify-content: center;
    }

    :host .emoji-search span {
      cursor: pointer;
      padding: 4px 3px 2px 4px;
      font-size: 24px;
    }

    :host .emoji-search span:hover {
      background-color: #ccc;
    }

  `]
            },] },
];
/** @nocollapse */
EmojiInputComponent.ctorParameters = () => [
    { type: EmojiService, },
];
EmojiInputComponent.propDecorators = {
    "textArea": [{ type: Input },],
    "popupAnchor": [{ type: Input },],
    "inputClass": [{ type: Input },],
    "searchClass": [{ type: Input },],
    "onEnter": [{ type: Input },],
    "model": [{ type: Input },],
    "autofocus": [{ type: Input },],
    "closeAfterSelection": [{ type: Input },],
    "group": [{ type: Input },],
    "placeHolder": [{ type: Input },],
    "modelChange": [{ type: Output },],
    "setPopupAction": [{ type: Output },],
    "blur": [{ type: Output },],
    "focus": [{ type: Output },],
    "keyup": [{ type: Output },],
    "emojiClick": [{ type: Output },],
    "textareaEl": [{ type: ViewChild, args: ['textareaEl',] },],
    "inputEl": [{ type: ViewChild, args: ['inputEl',] },],
    "clickedOutside": [{ type: HostListener, args: ['document:click', ['$event'],] },],
};
function EmojiInputComponent_tsickle_Closure_declarations() {
    /** @type {!Array<{type: !Function, args: (undefined|!Array<?>)}>} */
    EmojiInputComponent.decorators;
    /**
     * @nocollapse
     * @type {function(): !Array<(null|{type: ?, decorators: (undefined|!Array<{type: !Function, args: (undefined|!Array<?>)}>)})>}
     */
    EmojiInputComponent.ctorParameters;
    /** @type {!Object<string,!Array<{type: !Function, args: (undefined|!Array<?>)}>>} */
    EmojiInputComponent.propDecorators;
    /** @type {?} */
    EmojiInputComponent.prototype.textArea;
    /** @type {?} */
    EmojiInputComponent.prototype.popupAnchor;
    /** @type {?} */
    EmojiInputComponent.prototype.inputClass;
    /** @type {?} */
    EmojiInputComponent.prototype.searchClass;
    /** @type {?} */
    EmojiInputComponent.prototype.onEnter;
    /** @type {?} */
    EmojiInputComponent.prototype.model;
    /** @type {?} */
    EmojiInputComponent.prototype.autofocus;
    /** @type {?} */
    EmojiInputComponent.prototype.closeAfterSelection;
    /** @type {?} */
    EmojiInputComponent.prototype.group;
    /** @type {?} */
    EmojiInputComponent.prototype.placeHolder;
    /** @type {?} */
    EmojiInputComponent.prototype.modelChange;
    /** @type {?} */
    EmojiInputComponent.prototype.setPopupAction;
    /** @type {?} */
    EmojiInputComponent.prototype.blur;
    /** @type {?} */
    EmojiInputComponent.prototype.focus;
    /** @type {?} */
    EmojiInputComponent.prototype.keyup;
    /** @type {?} */
    EmojiInputComponent.prototype.emojiClick;
    /** @type {?} */
    EmojiInputComponent.prototype.textareaEl;
    /** @type {?} */
    EmojiInputComponent.prototype.inputEl;
    /** @type {?} */
    EmojiInputComponent.prototype.input;
    /** @type {?} */
    EmojiInputComponent.prototype.filterEmojis;
    /** @type {?} */
    EmojiInputComponent.prototype.filteredEmojis;
    /** @type {?} */
    EmojiInputComponent.prototype.allEmojis;
    /** @type {?} */
    EmojiInputComponent.prototype.popupOpen;
    /** @type {?} */
    EmojiInputComponent.prototype.lastCursorPosition;
    /** @type {?} */
    EmojiInputComponent.prototype.emojiService;
}
//# sourceMappingURL=emoji-input.component.js.map